@@ -77,13 +77,13 @@ for CC in ["gcc", "clang"]:
                 "USE_OPENSSL=1",
                 "USE_SYSTEMD=1",
                 "USE_WURFL=1",
-                "WURFL_INC=contrib/wurfl",
-                "WURFL_LIB=contrib/wurfl",
+                "WURFL_INC=addons/wurfl/dummy",
+                "WURFL_LIB=addons/wurfl/dummy",
                 "USE_DEVICEATLAS=1",
-                "DEVICEATLAS_SRC=contrib/deviceatlas",
-                "EXTRA_OBJS=contrib/prometheus-exporter/service-prometheus.o",
+                "DEVICEATLAS_SRC=addons/deviceatlas/dummy",
+                "USE_PROMEX=1",
                 "USE_51DEGREES=1",
-                "51DEGREES_SRC=contrib/51d/src/pattern",
+                "51DEGREES_SRC=addons/51degrees/dummy/pattern",
             ],
         }
     )
@@ -104,14 +104,13 @@ for CC in ["gcc", "clang"]:
     for ssl in [
         "stock",
         "OPENSSL_VERSION=1.0.2u",
-        "OPENSSL_VERSION=3.0.2",
         "LIBRESSL_VERSION=2.9.2",
-        "LIBRESSL_VERSION=3.5.1",
-#        "BORINGSSL=yes",
+        "LIBRESSL_VERSION=3.3.3",
+        "BORINGSSL=yes",
     ]:
         flags = ["USE_OPENSSL=1"]
-        if "OPENSSL_VERSION=3.0." in ssl:
-            flags.append('DEBUG_CFLAGS="-g -Wno-deprecated-declarations"')
+        if ssl == "BORINGSSL=yes":
+            flags.append("USE_QUIC=1")
         if ssl != "stock":
             flags.append("SSL_LIB=${HOME}/opt/lib")
             flags.append("SSL_INC=${HOME}/opt/include")
@@ -146,13 +145,13 @@ matrix.append(
             "USE_OPENSSL=1",
             "USE_SYSTEMD=1",
             "USE_WURFL=1",
-            "WURFL_INC=contrib/wurfl",
-            "WURFL_LIB=contrib/wurfl",
+            "WURFL_INC=addons/wurfl/dummy",
+            "WURFL_LIB=addons/wurfl/dummy",
             "USE_DEVICEATLAS=1",
-            "DEVICEATLAS_SRC=contrib/deviceatlas",
-            "EXTRA_OBJS=contrib/prometheus-exporter/service-prometheus.o",
+            "DEVICEATLAS_SRC=addons/deviceatlas/dummy",
+            "USE_PROMEX=1",
             "USE_51DEGREES=1",
-            "51DEGREES_SRC=contrib/51d/src/pattern",
+            "51DEGREES_SRC=addons/51degrees/dummy/pattern",
         ],
     }
 )
