@@ -68,6 +68,11 @@
 #define POOL_LINK(pool, item) ((void **)(item))
 #endif
 
+/* A special pointer for the pool's free_list that indicates someone is
+ * currently manipulating it. Serves as a short-lived lock.
+ */
+#define POOL_BUSY ((void *)1)
+
 #define POOL_AVG_SAMPLES 1024
 
 /* possible flags for __pool_alloc() */
@@ -86,19 +91,8 @@ struct pool_cache_item {
 	struct list by_lru;  /* link to objects by LRU order */
 };
 
-struct pool_free_list {
-	void **free_list;
-	uintptr_t seq;
-};
-
-/* Note below, in case of lockless pools, we still need the lock only for
- * the flush() operation.
- */
 struct pool_head {
 	void **free_list;
-#ifdef CONFIG_HAP_LOCKLESS_POOLS
-	uintptr_t seq;
-#endif
 	__decl_thread(HA_SPINLOCK_T lock); /* the spin lock */
 	unsigned int used;	/* how many chunks are currently in use */
 	unsigned int needed_avg;/* floating indicator between used and allocated */
@@ -109,6 +103,7 @@ struct pool_head {
 	unsigned int flags;	/* MEM_F_* */
 	unsigned int users;	/* number of pools sharing this zone */
 	unsigned int failed;	/* failed allocations */
+	/* 32-bit hole here */
 	struct list list;	/* list of all known pools */
 	char name[12];		/* name of the pool */
 #ifdef CONFIG_HAP_POOLS
