@@ -29,25 +29,33 @@ const char *trace_flt_id = "trace filter";
 
 struct flt_ops trace_ops;
 
+#define TRACE_F_QUIET       0x00000001
+#define TRACE_F_RAND_FWD    0x00000002
+#define TRACE_F_HEXDUMP     0x00000004
+
 struct trace_config {
 	struct proxy *proxy;
 	char         *name;
-	int           rand_parsing;
-	int           rand_forwarding;
-	int           hexdump;
+	unsigned int  flags;
 };
 
 #define FLT_TRACE(conf, fmt, ...)						\
-	fprintf(stderr, "%d.%06d [%-20s] " fmt "\n",			\
-		(int)now.tv_sec, (int)now.tv_usec, (conf)->name,	\
-		##__VA_ARGS__)
+	do {									\
+		if (!(conf->flags & TRACE_F_QUIET))				\
+			fprintf(stderr, "%d.%06d [%-20s] " fmt "\n",		\
+				(int)now.tv_sec, (int)now.tv_usec, (conf)->name,\
+				##__VA_ARGS__);					\
+	} while (0)
 
-#define FLT_STRM_TRACE(conf, strm, fmt, ...)						\
-	fprintf(stderr, "%d.%06d [%-20s] [strm %p(%x) 0x%08x 0x%08x] " fmt "\n",	\
-		(int)now.tv_sec, (int)now.tv_usec, (conf)->name,			\
-		strm, (strm ? ((struct stream *)strm)->uniq_id : ~0U),			\
-		(strm ? strm->req.analysers : 0), (strm ? strm->res.analysers : 0),	\
-		##__VA_ARGS__)
+#define FLT_STRM_TRACE(conf, strm, fmt, ...)								\
+	do {												\
+		if (!(conf->flags & TRACE_F_QUIET))							\
+			fprintf(stderr, "%d.%06d [%-20s] [strm %p(%x) 0x%08x 0x%08x] " fmt "\n",	\
+				(int)now.tv_sec, (int)now.tv_usec, (conf)->name,			\
+				strm, (strm ? ((struct stream *)strm)->uniq_id : ~0U),			\
+				(strm ? strm->req.analysers : 0), (strm ? strm->res.analysers : 0),	\
+				##__VA_ARGS__);								\
+	} while (0)
 
 
 static const char *
@@ -135,8 +143,7 @@ trace_htx_hexdump(struct htx *htx, unsigned int offset, unsigned int len)
 		}
 
 		v = htx_get_blk_value(htx, blk);
-		v.ptr += offset;
-		v.len -= offset;
+		v = istadv(v, offset);
 		offset = 0;
 
 		if (v.len > len)
@@ -188,14 +195,14 @@ trace_init(struct proxy *px, struct flt_conf *fconf)
 	fconf->flags |= FLT_CFG_FL_HTX;
 	fconf->conf = conf;
 
-	FLT_TRACE(conf, "filter initialized [read random=%s - fwd random=%s - hexdump=%s]",
-	      (conf->rand_parsing ? "true" : "false"),
-	      (conf->rand_forwarding ? "true" : "false"),
-	      (conf->hexdump ? "true" : "false"));
+	FLT_TRACE(conf, "filter initialized [quiet=%s - fwd random=%s - hexdump=%s]",
+		  ((conf->flags & TRACE_F_QUIET) ? "true" : "false"),
+		  ((conf->flags & TRACE_F_RAND_FWD) ? "true" : "false"),
+		  ((conf->flags & TRACE_F_HEXDUMP) ? "true" : "false"));
 	return 0;
 }
 
-/* Free ressources allocated by the trace filter. */
+/* Free resources allocated by the trace filter. */
 static void
 trace_deinit(struct proxy *px, struct flt_conf *fconf)
 {
@@ -227,7 +234,7 @@ trace_init_per_thread(struct proxy *px, struct flt_conf *fconf)
 	return 0;
 }
 
-/* Free ressources allocate by the trace filter for each thread. */
+/* Free resources allocate by the trace filter for each thread. */
 static void
 trace_deinit_per_thread(struct proxy *px, struct flt_conf *fconf)
 {
@@ -458,7 +465,7 @@ trace_http_payload(struct stream *s, struct filter *filter, struct http_msg *msg
 	struct trace_config *conf = FLT_CONF(filter);
 	int ret = len;
 
-	if (ret && conf->rand_forwarding) {
+	if (ret && (conf->flags & TRACE_F_RAND_FWD)) {
 		unsigned int data = trace_get_htx_datalen(htxbuf(&msg->chn->buf), offset, len);
 
 		if (data) {
@@ -474,7 +481,7 @@ trace_http_payload(struct stream *s, struct filter *filter, struct http_msg *msg
 		   channel_label(msg->chn), proxy_mode(s), stream_pos(s),
 		   offset, len, ret);
 
-	 if (conf->hexdump)
+	 if (conf->flags & TRACE_F_HEXDUMP)
 		 trace_htx_hexdump(htxbuf(&msg->chn->buf), offset, ret);
 
 	 if (ret != len)
@@ -526,7 +533,7 @@ trace_tcp_payload(struct stream *s, struct filter *filter, struct channel *chn,
 	int ret = len;
 
 	if (s->flags & SF_HTX) {
-		if (ret && conf->rand_forwarding) {
+		if (ret && (conf->flags & TRACE_F_RAND_FWD)) {
 			unsigned int data = trace_get_htx_datalen(htxbuf(&chn->buf), offset, len);
 
 			if (data) {
@@ -542,12 +549,12 @@ trace_tcp_payload(struct stream *s, struct filter *filter, struct channel *chn,
 			       channel_label(chn), proxy_mode(s), stream_pos(s),
 			       offset, len, ret);
 
-		if (conf->hexdump)
+		if (conf->flags & TRACE_F_HEXDUMP)
 			trace_htx_hexdump(htxbuf(&chn->buf), offset, ret);
 	}
 	else {
 
-		if (ret && conf->rand_forwarding)
+		if (ret && (conf->flags & TRACE_F_RAND_FWD))
 			ret = ha_random() % (ret+1);
 
 		FLT_STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s) - "
@@ -556,7 +563,7 @@ trace_tcp_payload(struct stream *s, struct filter *filter, struct channel *chn,
 			       channel_label(chn), proxy_mode(s), stream_pos(s),
 			       offset, len, ret);
 
-		if (conf->hexdump)
+		if (conf->flags & TRACE_F_HEXDUMP)
 			trace_raw_hexdump(&chn->buf, offset, ret);
 	}
 
@@ -614,12 +621,12 @@ parse_trace_flt(char **args, int *cur_arg, struct proxy *px,
 		return -1;
 	}
 	conf->proxy = px;
-
-	if (!strcmp(args[pos], "trace")) {
+	conf->flags = 0;
+	if (strcmp(args[pos], "trace") == 0) {
 		pos++;
 
 		while (*args[pos]) {
-			if (!strcmp(args[pos], "name")) {
+			if (strcmp(args[pos], "name") == 0) {
 				if (!*args[pos + 1]) {
 					memprintf(err, "'%s' : '%s' option without value",
 						  args[*cur_arg], args[pos]);
@@ -632,12 +639,14 @@ parse_trace_flt(char **args, int *cur_arg, struct proxy *px,
 				}
 				pos++;
 			}
-			else if (!strcmp(args[pos], "random-parsing"))
-				conf->rand_parsing = 1;
-			else if (!strcmp(args[pos], "random-forwarding"))
-				conf->rand_forwarding = 1;
-			else if (!strcmp(args[pos], "hexdump"))
-				conf->hexdump = 1;
+			else if (strcmp(args[pos], "quiet") == 0)
+				conf->flags |= TRACE_F_QUIET;
+			else if (strcmp(args[pos], "random-parsing") == 0)
+				continue; // ignore
+			else if (strcmp(args[pos], "random-forwarding") == 0)
+				conf->flags |= TRACE_F_RAND_FWD;
+			else if (strcmp(args[pos], "hexdump") == 0)
+				conf->flags |= TRACE_F_HEXDUMP;
 			else
 				break;
 			pos++;
